/**
 * $Id: viewimage.c,v 1.14 2003/08/28 07:12:01 chakie Exp $
 * Modified for Dingoo by Stephanie Maks
 **/

#include "SDL.h"
#include "SDL_image.h"
#include "SDL_rotozoom.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <dirent.h>
#include "dingoo.h"

/**
 * a single node in the list of images.
 **/
struct list_node {
	char * m_filename;
	int m_index;
	struct list_node * m_next;
	struct list_node * m_prev;
};

// the list pointers
static struct list_node * list_head = NULL;
static struct list_node * list_tail = NULL;

// the current image
static struct list_node * current_node = NULL;

// the number of images in the list
static int count = 0;

static SDL_Surface * image	  = NULL;
static SDL_Surface * rotated	= NULL;
static SDL_Surface * next_image = NULL;
static SDL_Surface * screen	 = NULL;

// global options
static int screen_width  = 320;
static int screen_height = 240;
// force fullscreen mode on for Dingoo
static int flags		 = SDL_ANYFORMAT | SDL_FULLSCREEN;
static int antialias	 = 0;
int degrees = 0;

void blit () {
	SDL_Rect dest;
	SDL_Surface * blitted;

	if ( rotated != NULL ) {
		blitted = rotated;
	} else {
		blitted = image;
	}

	dest.x = (screen->w - blitted->w) / 2;
	dest.y = (screen->h - blitted->h) / 2;
	dest.w = blitted->w;
	dest.h = blitted->h;	
	
	SDL_FillRect (screen, NULL, SDL_MapRGB (screen->format, 0, 0, 0));

	// blit it out
	SDL_BlitSurface (blitted, 0, screen, &dest);
	SDL_Flip (screen);
}

void rotate () {
	float zoom = 1.0;
	// no rotation at all, a going back to the original?
	if ( degrees == 0 || degrees % 360 == 0 ) {
		if ( rotated != NULL ) {
			SDL_FreeSurface ( rotated );
			rotated = NULL;
		}
		blit ();
		return;
	}
	
	// check scaling of rotated product
	if ( degrees % 180 != 0 ) {
		if ( image->w > image->h) {
			zoom = (float)screen->h / (float)image->w;
		} else if ( image->h > image->w ) {
			zoom = (float)screen->w / (float)image->h;
		}
//		printf("\t\tNew zoom is %.2f\n", zoom);
	}

	if ( rotated != NULL ) {
		SDL_FreeSurface ( rotated );
		rotated = NULL;
	}
//	printf("Rotating the image to %d degrees.", degrees);
	rotated = rotozoomSurface ( image, (double)degrees, zoom, antialias );
	blit ();
}


SDL_Surface * loadImage (char * filename) {
	struct stat fileinfo;
	SDL_Surface * new_image;
	SDL_Surface * image_tmp;

	if ( stat ( filename, &fileinfo ) == -1 ) {
		printf ( "error reading file %s: %s\n", filename, strerror (errno) );
		exit ( EXIT_FAILURE );
	}
		
	new_image = IMG_Load( filename );
	if(!new_image) {
		printf ("loadImage: %s\n", IMG_GetError());
		SDL_Quit();
		return NULL;
	}

	printf ("Loaded %s ", filename);
	if ( new_image->w > screen->w || new_image->h > screen->h ) {
		double rot=0.0, scale=1.0;
		if ( new_image->h > new_image->w ) {
			// too tall - rotate 90 degrees and scale image hieght to screen width
			scale = (double)screen->w / (double)new_image->h;
			rot= -90.0;
		} else {
			// too wide or just right, scale based on width only
			scale = (double)screen->w / (double)new_image->w;
		}
		if ( rot != 0.0 ) printf(" R ");

		image_tmp = rotozoomSurface( new_image, rot, scale, antialias );
		SDL_FreeSurface ( new_image );
		new_image = image_tmp;
	}
	printf ( ", %d x %d\n", new_image->w, new_image->h);
	return new_image;
}

int next () {
	if ( current_node == NULL && list_head != NULL ) {
		current_node = list_head;
	} else {
		if ( current_node == list_tail ) {
			return 0;
		}
		current_node = current_node->m_next;
	}
	if ( next_image != NULL ) {
		SDL_FreeSurface ( image );
		image = next_image;
	} else {
		SDL_FreeSurface ( image );
		image = loadImage ( current_node->m_filename );
	}
	if ( rotated != NULL ) {
		SDL_FreeSurface ( rotated );
		rotated = NULL;
	}
	degrees = 0;
	blit ();
	if ( current_node->m_next != NULL ) {
		next_image = loadImage ( current_node->m_next->m_filename );
	}
	else {
		next_image = NULL;
	}
	return 1;
}


int previous () {
	if ( current_node == list_head ) return 0;

	current_node = current_node->m_prev;
	if ( next_image != NULL ) SDL_FreeSurface ( next_image );

	if ( rotated != NULL ) {
		SDL_FreeSurface ( rotated );
		rotated = NULL;
	}
	next_image = image;
	image = loadImage ( current_node->m_filename );
	degrees = 0;
	blit ();
	return 1;
}

void addToList (char * filename) {
	count++;
	struct list_node * node = (struct list_node *)malloc ( sizeof (struct list_node) );

	node->m_filename = strdup ( filename );
	node->m_index = count;
	node->m_next = NULL;
	node->m_prev = list_tail;

	if ( list_head == NULL ) {
		list_head = node;
		list_tail = node;
	} else {
		list_tail->m_next = node;
		list_tail = node;
	}
}

void readFilenames (int argc, char **argv) {
	int index;
	char * fileName;
	struct stat info;

	for ( index = 0; index < argc; index++ ) {
		fileName = argv[index];
		if ( stat ( fileName, &info ) == -1 ) {
			printf ( "could not open %s: %s\n", fileName, strerror (errno) );
			continue;
		}

		if ( S_ISREG ( info.st_mode ) ) {
			addToList ( fileName );
		} else if ( S_ISDIR ( info.st_mode ) ) {
			printf ( "%s is a dir, ignoring for now\n", fileName );
		} else {
			printf ( "%s is something we don't know about, ignoring\n", fileName );
		}
	}

	if ((argc == 1) && (count == 1)) {
		char *ff = strrchr(fileName, '/');
		char *dirName = strdup(fileName);
		dirName[(ff - fileName) + 1] = '\0'; 
		char* suffix;
		suffix=strrchr(ff,'.') + 1;
		struct dirent *d;
		DIR *dir;
		char buf[256];
		dir = opendir(dirName);
		while((d = readdir(dir)) != NULL) {
			if(strstr(d->d_name, suffix)) {
				sprintf(buf, "%s%s", dirName, d->d_name);
				if((strcmp(fileName, buf)) != 0) addToList(buf);
			}
		}
	}
	printf("Loaded %d images in total.\n", count);
}

int main(int argc, char **argv) {
	SDL_Event event;
	SDLKey key;
	int done=0;
	
	if(argc < 2)	{
		printf("Usage: %s filename [filename] ...\n",*argv);
		return 1;
	}

	readFilenames ( argc - 1, argv + 1 );

	if ( list_head == NULL ) {
		printf ( "no valid images found\n" );
		exit ( EXIT_SUCCESS );
	}
	
	if(SDL_Init(SDL_INIT_VIDEO)==-1) {
		printf("SDL_Init: %s\n", SDL_GetError());
		return 1;
	}
	
	screen = SDL_SetVideoMode ( screen_width, screen_height, 0, flags );
	SDL_ShowCursor(0);

	if (!screen)	{
		printf("SDL_SetVideoMode: %s\n", SDL_GetError());
		SDL_Quit();
		return 1;
	}
	
	next ();
		
	while (!done && SDL_WaitEvent(&event) != -1) {
		switch(event.type) {
		case SDL_KEYDOWN:
			key=event.key.keysym.sym;
			switch (key)  {
			case DINGOO_BUTTON_RIGHT:
				if ( ! next () ) continue;
				break;
			case DINGOO_BUTTON_LEFT:
				previous ();
				break;
			case DINGOO_BUTTON_L:
				degrees += 90;
				rotate ();
				break;
			case DINGOO_BUTTON_R:
				degrees -= 90;
				rotate ();
				break;
			case DINGOO_BUTTON_SELECT:
			case SDLK_q:
				done = 1;
			default: 
				continue;
			}
			break;
		case SDL_VIDEOEXPOSE:
			blit ();
			break;
		case SDL_QUIT:
			done = 1;
			break;
		default:
			break;
		}
	}
	SDL_FreeSurface ( image );
	if ( next_image != NULL ) SDL_FreeSurface ( next_image );
	SDL_Quit();
	return 0;
}

